;; Generated package description from auth-source-xoauth2-plugin.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "auth-source-xoauth2-plugin" "0.2.1" "authentication source plugin for xoauth2" '((emacs "28.1") (oauth2 "0.17")) :commit "08c5cf49a9ce69cd2a39f65f91abfeb7f6ae7772" :url "https://elpa.gnu.org/packages/auth-source-xoauth2-plugin.html" :authors '(("Xiyue Deng" . "manphiz@gmail.com")) :maintainer '("Xiyue Deng" . "manphiz@gmail.com"))
